@@ -1,29 +1,16 @@
-### read.nexus.R (2006-08-09)
+### read.nexus.R (2006-11-07)
 ###
 ###     Read Tree File in Nexus Format
 ###
 ### Copyright 2003-2006 Emmanuel Paradis
 ###
-### This file is part of the `ape' library for R and related languages.
-### It is made available under the terms of the GNU General Public
-### License, version 2, or at your option, any later version,
-### incorporated herein by reference.
-###
-### This program is distributed in the hope that it will be
-### useful, but WITHOUT ANY WARRANTY; without even the implied
-### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-### PURPOSE.  See the GNU General Public License for more
-### details.
-###
-### You should have received a copy of the GNU General Public
-### License along with this program; if not, write to the Free
-### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-### MA 02111-1307, USA
+### This file is part of the R-package `ape'.
+### See the file ../COPYING for licensing issues.
 
 clado.build <- function(tp) {
     add.internal <- function() {
         edge[j, 1] <<- current.node
-        node <<- node - 1
+        node <<- node + 1
         edge[j, 2] <<- current.node <<- node
         j <<- j + 1
     }
@@ -37,12 +24,12 @@ clado.build <- function(tp) {
     }
     go.down <- function() {
         l <- which(edge[, 2] == current.node)
-        node.label[-current.node] <<- tpc[k]
+        node.label[current.node - nb.tip] <<- tpc[k]
         k <<- k + 1
         current.node <<- edge[l, 1]
     }
     if (!length(grep(",", tp))) {
-        obj <- list(edge = matrix(c("-1", "1"), 1, 2))
+        obj <- list(edge = matrix(c(2, 1), 1, 2))
         tp <- unlist(strsplit(tp, "[\\(\\);]"))
         obj$tip.label <- tp[2]
         if (length(tp) == 3) obj$node.label <- tp[3]
@@ -65,13 +52,14 @@ clado.build <- function(tp) {
     tip.label <- character(nb.tip)
 
     edge <- matrix(NA, nb.edge, 2)
-    edge[nb.edge, 1] <- 0  # see comment above
-    edge[nb.edge, 2] <- -1 #
-    node <- -1                              # node number
-    current.node <- node
-    j <- 1                                  # index of the line number of edge
-    k <- 1                                  # index of the line number of tpc
-    tip <- 1                                # tip number
+    current.node <- node <- nb.tip + 1 # node number
+    edge[nb.edge, 1] <- 0    # see comment above
+    edge[nb.edge, 2] <- node #
+
+    ## j: index of the line number of edge
+    ## k: index of the line number of tpc
+    ## tip: tip number
+    j <- k <- tip <- 1
 
     for (i in 2:nsk) {
         if (skeleton[i] == "(") add.internal()      # add an internal branch (on top)
@@ -88,10 +76,8 @@ clado.build <- function(tp) {
     }
 #    if(node.label[1] == "NA") node.label[1] <- ""
     edge <- edge[-nb.edge, ]
-    mode(edge) <- "character"
-    obj <- list(edge = edge,
-                tip.label = tip.label,
-                node.label = node.label)
+    obj <- list(edge = edge, tip.label = tip.label,
+                Nnode = nb.node, node.label = node.label)
     obj$node.label <- if (all(obj$node.label == "NA")) NULL else gsub("^NA", "", obj$node.label)
     class(obj) <- "phylo"
     return(obj)
@@ -103,26 +89,28 @@ read.nexus <- function(file, tree.names = NULL)
     ## first remove all the comments
     LEFT <- grep("\\[", X)
     RIGHT <- grep("\\]", X)
-    for (i in length(LEFT):1) {
-        if (LEFT[i] == RIGHT[i]) {
-            X[LEFT[i]] <- gsub("\\[.*\\]", "", X[LEFT[i]])
-        } else {
-            X[LEFT[i]] <- gsub("\\[.*", "", X[LEFT[i]])
-            X[RIGHT[i]] <- gsub(".*\\]", "", X[RIGHT[i]])
-            if (LEFT[i] < RIGHT[i] - 1) X <- X[-((LEFT[i] + 1):(RIGHT[i] - 1))]
+    if (length(LEFT)) {
+        for (i in length(LEFT):1) {
+            if (LEFT[i] == RIGHT[i]) {
+                X[LEFT[i]] <- gsub("\\[.*\\]", "", X[LEFT[i]])
+            } else {
+                X[LEFT[i]] <- gsub("\\[.*", "", X[LEFT[i]])
+                X[RIGHT[i]] <- gsub(".*\\]", "", X[RIGHT[i]])
+                if (LEFT[i] < RIGHT[i] - 1) X <- X[-((LEFT[i] + 1):(RIGHT[i] - 1))]
+            }
         }
     }
-    X <- gsub("[Ee][Nn][Dd][Bb][Ll][Oo][Cc][Kk];", "END;", X)
-    endblock <- grep("[Ee][Nn][Dd];", X)
+    X <- gsub("ENDBLOCK;", "END;", X, ignore.case = TRUE)
+    endblock <- grep("END;", X, ignore.case = TRUE)
     semico <- grep(";", X)
-    i1 <- grep("[Bb][Ee][Gg][Ii][Nn] [Tt][Rr][Ee][Ee][Ss];", X)
-    i2 <- grep("[Tt][Rr][Aa][Nn][Ss][Ll][Aa][Tt][Ee]", X)
+    i1 <- grep("BEGIN TREES;", X, ignore.case = TRUE)
+    i2 <- grep("TRANSLATE", X, ignore.case = TRUE)
     translation <- FALSE
     if (length(i2) == 1) if (i2 > i1) translation <- TRUE
     if (translation) {
         end <- semico[semico > i2][1]
         x <- paste(X[i2:end], sep = "", collapse = "")
-        x <- gsub("[Tt][Rr][Aa][Nn][Ss][Ll][Aa][Tt][Ee]", "", x)
+        x <- gsub("TRANSLATE", "", x, ignore.case = TRUE)
         x <- unlist(strsplit(x, "[,; \t]"))
         x <- x[x != ""]
         TRANS <- matrix(x, ncol = 2, byrow = TRUE)
@@ -166,7 +154,7 @@ read.nexus <- function(file, tree.names = NULL)
                 trees[[i]]$edge[j] <- as.character(as.numeric(trees[[i]]$edge[j]) + 1)
             ## Check a second time and if there is still a problem...!!!
             if(sum(trees[[i]]$edge[, 1] == "-1") == 1)
-              stop("There is apparently two root edges in your file:\cannot read tree file")
+              stop("There is apparently two root edges in your file: cannot read tree file")
         }
     }
     if (nb.tree == 1) trees <- trees[[1]] else {
