@@ -1,31 +1,25 @@
-### write.dna.R (2003-12-23)
+### write.dna.R (2003-11-19)
 ###
 ###     Write DNA Sequences in a File
 ###
-### Copyright 2003 Emmanuel Paradis
+### Copyright 2003-2006 Emmanuel Paradis
 ###
-### This file is part of the `ape' library for R and related languages.
-### It is made available under the terms of the GNU General Public
-### License, version 2, or at your option, any later version,
-### incorporated herein by reference.
-###
-### This program is distributed in the hope that it will be
-### useful, but WITHOUT ANY WARRANTY; without even the implied
-### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-### PURPOSE.  See the GNU General Public License for more
-### details.
-###
-### You should have received a copy of the GNU General Public
-### License along with this program; if not, write to the Free
-### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-### MA 02111-1307, USA
+### This file is part of the R-package `ape'.
+### See the file ../COPYING for licensing issues.
 
 write.dna <- function(x, file, format = "interleaved", append = FALSE,
                       nbcol = 6, colsep = " ", colw = 10, indent = NULL,
                       blocksep = 1)
 {
     format <- match.arg(format, c("interleaved", "sequential", "fasta"))
-    N <- length(x)
+    if (is.matrix(x)) {
+        N <- dim(x)[1]
+        xx <- vector("list", N)
+        for (i in 1:N) xx[[i]] <- x[i, ]
+        names(xx) <- rownames(x)
+        x <- xx
+        rm(xx)
+    } else N <- length(x)
     if (is.null(names(x))) names(x) <- as.character(1:N)
     if (is.null(indent)) {
         indent <- if (format %in% c("interleaved", "sequential")) 10 else  0
